/*
** Carousel2D v1.0
** Ítalo Alves Bezerra do Nascimento
** contact@italonascimento.com
*/

function carousel2d(element, options) {
	//setting default values is the options are nor specified
	var settings = {
		time 			:typeof options.time 			!= 'undefined' ? options.time 			: 300,
		wrapper 		:typeof options.wrapper 		!= 'undefined' ? options.wrapper 		: '.wrapper',
		slider 			:typeof options.slider 			!= 'undefined' ? options.slider 		: '.slider',
		items 			:typeof options.items 			!= 'undefined' ? options.items 			: '.items',
		arrowPrev  		:typeof options.arrowPrev 		!= 'undefined' ? options.arrowPrev 		: '.prev',
		arrowNext  		:typeof options.arrowNext 		!= 'undefined' ? options.arrowNext 		: '.next',
		disabledOpacity	:typeof options.disabledOpacity	!= 'undefined' ? options.disabledOpacity: 0.5
	}
	
	var root 		= element,
		isAnimating = false,
		wrapper 	= nut(settings.wrapper, root)[0],
		slider 		= nut(settings.slider, wrapper)[0],
		prev 		= nut(settings.arrowPrev)[0],
		next 		= nut(settings.arrowNext)[0],
		items 		= nut(settings.items, slider),
		single		= items[0],
		singleWidth = single.offsetWidth,
		visible 	= Math.ceil(wrapper.offsetWidth / singleWidth),
		currentPage = 1,
		pages 		= Math.ceil(items.length / visible);
	
	// needed css config
	wrapper.style.overflow 	= 'hidden';
	wrapper.style.position 	= 'relative';
	slider.style.width		= (singleWidth * items.length)+'px';
	slider.style.left 		= 0;
	slider.style.position 	= 'absolute';
	
	checkArrows();
	
	prev.onclick = function () {
		if(currentPage > 1){
			root.goto(currentPage - 1);
		}
	}
	
	next.onclick = function () {
		if(currentPage < pages){
			root.goto(currentPage + 1);
		}
	}
	
	root.goto = function(page) {
		if(!isAnimating && page >= 1 && page <= pages){
			isAnimating = true;
			
			var dir = page < currentPage ? 1 : -1,
				n = Math.abs(currentPage - page),
				leftMove = singleWidth * dir * visible * n;
			
			currentPage = page;
			root.onStart(currentPage);
			
			checkArrows();
			
			var oldLeft = parseFloat(slider.style.left.replace('px', '')),
				currentLeft = oldLeft,
				newLeft = currentLeft + leftMove;
				
			emile(slider, 'left:'+newLeft+'px', {
				duration:settings.time, 
				after:function(){
					isAnimating = false;
					root.onComplete(currentPage);
				}
			});
		}
	}
	
	root.onStart = function(page){}
	root.onComplete = function(page){}
	
	function checkArrows(){
		// solving the ie problem on setting opacity to not positioned elements (hasLayout = false)
		prev.style.zoom = 1
		next.style.zoom = 1
		
		if(currentPage == 1){
			prev.style.opacity = settings.disabledOpacity;
			prev.style.MozOpacity = settings.disabledOpacity;
			prev.style.KhtmlOpacity = settings.disabledOpacity;
			prev.style.filter = 'alpha(opacity=' + (settings.disabledOpacity * 100) + ')';
			
			prev.style.cursor = 'default';
		}else{
			prev.style.opacity = 1;
			prev.style.MozOpacity = 1;
			prev.style.KhtmlOpacity = 1;
			prev.style.filter = 'alpha(opacity=100)';
			
			prev.style.cursor = 'pointer';
		}
		
		if(currentPage == pages){
			next.style.opacity = settings.disabledOpacity;
			next.style.MozOpacity = settings.disabledOpacity;
			next.style.KhtmlOpacity = settings.disabledOpacity;
			next.style.filter = 'alpha(opacity=' + (settings.disabledOpacity * 100) + ')';
			
			next.style.cursor = 'default';
		}else{
			next.style.opacity = 1;
			next.style.MozOpacity = 1;
			next.style.KhtmlOpacity = 1;
			next.style.filter = 'alpha(opacity=100)';
			
			next.style.cursor = 'pointer';
		}
	}
	
	root.reinitialize = function(){
		items 		= nut(settings.items, slider);
		single		= items[0];
		singleWidth = single.offsetWidth;
		visible 	= Math.ceil(wrapper.offsetWidth / singleWidth);
		currentPage = 1;
		pages 		= Math.ceil(items.length / visible);
		
		slider.style.left = 0;
		checkArrows();
	}
	
	return root;
}